# Уроборос

Плагин к Picodata для репликации данных между двумя кластерами одного приложения, созданного при помощи фреймворка vshard.

[RFC](https://docs.google.com/document/d/1EvWjFd-7sFDTNGYRFB5OE79WtA7mMkwehYYDWrpzB1s/edit#heading=h.p9yvbnnvit8c)

## Сборка

(Описание создания локального тестового окружения находится в [файле](test-clusters.md))

### Собрать уроборос

```sh
make build
```

### Скопировать плагин и манифест в директорию плагина

Ссылки не получится сделать из-за [бага в Пикодате](https://git.picodata.io/picodata/picodata/picodata/-/issues/810)

```sh
mkdir -p plugins/uroboros
cp ./target/debug/liburoboros.so ./plugins/uroboros/<версия>/liburoboros.so
cp manifest.yml ./plugins/uroboros/<версия>/manifest.yaml
cp -r migrations/* plugins/uroboros/<версия>
```

где `<версия>` - последний тег. Папка `plugins/uroboros` - наш дистрибутив.

При необходимости отредактировать `plugins/<версия>/manifest.yml` - изменить адреса кластеров тарантула и прочее.

### Сборка под RockyLinux

Временное решение пока: докер с пробросом туда ssh сокета. Модифицируйте, если у вас другой сокет у агента или, например, ключ не по умолчанию.

```sh
ssh-agent -s
ssh-add
make docker_rocky_build
```

##  Тесты и тестовое окружение

### Требования

Для запуска тестового окружения необходимо наличие следующих программ:

- установленного приложения `tarantool`, собранного из [форка](https://git.picodata.io/picodata/tarantool/-/tree/2.11.2-picodata?ref_type=heads) с установкой `tarantoolctl` (-DENABLE_DIST=ON);
- установленного приложения [cartridge-cli](https://github.com/tarantool/cartridge-cli)
- наличия склонированного репозитория [Picodata, ветка `popov/uroboros-work-only-here`](https://git.picodata.io/picodata/picodata/picodata/-/tree/popov/uroboros-work-only-here?ref_type=heads) в папку `picodata` уровнем выше папки текущего проекта. Программа должна быть собрана для отладки (`cargo build`);
- GNU make;
- python и pip;
 
Т.к. тарантульные кластера источника и приемника запускаются в докере, то для сборки роли перед запуском необходимо предварительно запустить `ssh-agent`:

```sh
ssh-agent -s
ssh-add
```

### Локальное тестовое окружение

Состоит из следующих компонентов:

- тарантульный кластер `left` с Cartridge-приложение и ролями `uroboros.roles.*`;
- тарантульный кластер `right` с Cartridge-приложение и ролями `uroboros.roles.*`;
- кластер из двух инстансов picodata: `one` и `two`.

Kластер piсodata расположен в папке [local-dev-clusters](local-dev-clusters).

Код Cartridge-приложения находится в папке [src](local-dev-clusters/src). Инструкция для сборки образа с кластером находится в корне проекта, в файле [Dockerfile.cartridge.test](Dockerfile.cartridge.test). Compose-файл для запуска кластеров тарантула расположен в [каталоге local-dev-clusters](local-dev-clusters/docker-compose.yml).

### Подготовка к тестированию

Установите [`poetry`](https://python-poetry.org/) и необходимые зависимости:

```sh
make python_test_deps_install
```

Соберите проект:

```sh
make build
```

Создайте тестовую папку артефактов:

```sh
make plugin_test_dir
```

### Тестирование sidecar-роли

```sh
make test_sidecar
```

### Интеграционные тесты  с использованием docker compose

Это способ запуска интеграционных тестов по умолчанию. Требует установленного `docker`.

Соберите образ для поднятия кластеров с Cartridge-приложением:

```sh
make build_docker_cluster
```

Запустите тесты в docker-окружении:

```sh
make test_plugin
```

### Интеграционные тесты  с использованием cartridge

Требует установленного [cartridge-cli](https://github.com/tarantool/cartridge-cli).

Подготовьте файлы для поднятия кластеров с Cartridge-приложением:

```sh
make build_cartridge_cluster
```

Запустите тесты:

```sh
TEST_RUN_CLUSTERS=cartridge make test_plugin
```

### Запуск локального окружения

1. (Пере)Создать окружение `docker`:

```sh
make dev_docker_create
```

или `cartridge`:

```sh
make dev_cartridge_create
```

При выполнении команды происходит остановка запущенного окружения, очистка данных, пересборка кода и создание новых кластеров.

2. Запустить окружение `docker`:

```sh
make dev_docker_start
```

или `cartridge`:

```sh
make dev_cartridge_start
```

После успешного запуска инстансы пикодаты доступны через `admin.sock` в соответствующих каталогах: `local-dev-clusters/one` и `local-dev-clusters/two`. Порты подключения к инстансам тарантула указаны в [docker-compose.yml](local-dev-clusters/docker-compose.yml).

2. Подсоединиться через unix сокет как админ

```sh
../picodata/target/debug/picodata admin local-dev-clusters/one/admin.sock
```

3. Запустить плагин

```lua
CREATE PLUGIN uroboros <версия>
ALTER PLUGIN uroboros MIGRATE TO <версия>
ALTER PLUGIN uroboros <версия> ADD SERVICE uroboros TO TIER default
ALTER PLUGIN uroboros <версия> ENABLE
```
 где `<версия>` - последний тег проекта.

 3. По окончании остановить окружение `docker`:

```sh
make dev_docker_stop
```

или `cartridge`:

```sh
make dev_cartridge_stop
```
