


from time import sleep
from conftest import TarantoolCluster

_1_SEC = 1

def test_space_create_ddl_nonreplicated(producer_cluster: TarantoolCluster, consumer_cluster: TarantoolCluster) -> None:
    space_name = "test_space_create_ddl_nonreplicated"
    space_format = [   
        {'name': 'id', 'type': 'unsigned'},
        {'name': 'name', 'type': 'string'},
        {'name': 'age', 'type': 'unsigned'},
    ]
    producer_cluster.create_space(space_name, space_format, is_sharded=False)
    sleep(_1_SEC)
    consumer_cluster.fetch_spaces()
    assert consumer_cluster.space_format(space_name) is None


def test_space_create_ddl_replicated(producer_cluster: TarantoolCluster, consumer_cluster: TarantoolCluster) -> None:
    space_name = "test_space_create_ddl_replicated"
    space_format = [   
        {'name': 'id', 'type': 'unsigned'},
        {'name': 'name', 'type': 'string'},
        {'name': 'age', 'type': 'unsigned'},
        {'name': 'bucket_id', 'type': 'unsigned'},
    ]
    producer_cluster.create_space(space_name, space_format)
    sleep(_1_SEC)
    consumer_cluster.fetch_spaces()
    assert consumer_cluster.space_format(space_name) == space_format


def test_space_drop_ddl_replicated(producer_cluster: TarantoolCluster, consumer_cluster: TarantoolCluster) -> None:
    space_name = "test_space_drop_ddl_replicated"
    space_format = [   
        {'name': 'id', 'type': 'unsigned'},
        {'name': 'name', 'type': 'string'},
        {'name': 'age', 'type': 'unsigned'},
        {'name': 'bucket_id', 'type': 'unsigned'},
    ]
    producer_cluster.create_space(space_name, space_format)
    sleep(_1_SEC * 2)
    consumer_cluster.fetch_spaces()
    assert consumer_cluster.space_format(space_name) == space_format
    producer_cluster.drop_space(space_name)
    sleep(_1_SEC * 2)
    consumer_cluster.fetch_spaces()
    assert consumer_cluster.space_format(space_name) is None
    
    
def test_space_add_field_replicated(producer_cluster: TarantoolCluster, consumer_cluster: TarantoolCluster) -> None:
    space_name = "test_space_add_field_replicated"
    space_format = [   
        {'name': 'id', 'type': 'unsigned'},
        {'name': 'name', 'type': 'string'},
        {'name': 'age', 'type': 'unsigned'},
        {'name': 'bucket_id', 'type': 'unsigned'},
    ]
    producer_cluster.create_space(space_name, space_format)
    sleep(_1_SEC)
    consumer_cluster.fetch_spaces()
    assert consumer_cluster.space_format(space_name) == space_format
    add_field_name = "age2"
    add_field_type = "unsigned" 
    space_format.append({"name": add_field_name, "type": add_field_type})
    producer_cluster.add_field(space_name, add_field_name, add_field_type)
    sleep(_1_SEC)
    consumer_cluster.fetch_spaces()
    assert consumer_cluster.space_format(space_name) == space_format
    

def test_space_remove_field_replicated(producer_cluster: TarantoolCluster, consumer_cluster: TarantoolCluster) -> None:
    space_name = "test_space_remove_field_replicated"
    space_format = [   
        {'name': 'id', 'type': 'unsigned'},
        {'name': 'name', 'type': 'string'},
        {'name': 'age', 'type': 'unsigned'},
        {'name': 'bucket_id', 'type': 'unsigned'},
    ]
    producer_cluster.create_space(space_name, space_format)
    sleep(_1_SEC)
    consumer_cluster.fetch_spaces()
    assert consumer_cluster.space_format(space_name) == space_format
    remove_field = space_format.pop(2)
    producer_cluster.remove_field(space_name, remove_field["name"])
    sleep(_1_SEC)
    consumer_cluster.fetch_spaces()
    assert consumer_cluster.space_format(space_name) == space_format
    
    
def test_index_create_replicated(producer_cluster: TarantoolCluster, consumer_cluster: TarantoolCluster) -> None:
    index_name = "test_index_create_replicated_index"
    space_name = "test_index_create_replicated"
    space_format = [   
        {'name': 'id', 'type': 'unsigned'},
        {'name': 'name', 'type': 'string'},
        {'name': 'age', 'type': 'unsigned'},
        {'name': 'bucket_id', 'type': 'unsigned'},
    ]
    producer_cluster.create_space(space_name, space_format)
    sleep(_1_SEC)
    consumer_cluster.fetch_spaces()
    consumer_cluster.fetch_indexes()
    assert consumer_cluster.space_format(space_name) == space_format
    assert consumer_cluster.space_indexes(space_name) == {"primary", "bucket_id"}
    producer_cluster.create_index(space_name, index_name, (2, "string"))
    sleep(_1_SEC)
    consumer_cluster.fetch_indexes()
    assert consumer_cluster.space_indexes(space_name) == {"primary", "bucket_id", index_name}
    
    
def test_index_drop_replicated(producer_cluster: TarantoolCluster, consumer_cluster: TarantoolCluster) -> None:
    index_name = "test_index_drop_replicated_index"
    space_name = "test_index_drop_replicated"
    space_format = [   
        {'name': 'id', 'type': 'unsigned'},
        {'name': 'name', 'type': 'string'},
        {'name': 'age', 'type': 'unsigned'},
        {'name': 'bucket_id', 'type': 'unsigned'},
    ]
    producer_cluster.create_space(space_name, space_format)
    sleep(_1_SEC)
    consumer_cluster.fetch_spaces()
    consumer_cluster.fetch_indexes()
    assert consumer_cluster.space_format(space_name) == space_format
    assert consumer_cluster.space_indexes(space_name) == {"primary", "bucket_id"}
    producer_cluster.create_index(space_name, index_name, (2, "string"))
    sleep(_1_SEC)
    consumer_cluster.fetch_indexes()
    assert consumer_cluster.space_indexes(space_name) == {"primary", "bucket_id", index_name}
    sleep(_1_SEC)
    producer_cluster.drop_index(space_name, index_name)
    sleep(_1_SEC)
    consumer_cluster.fetch_indexes()
    assert consumer_cluster.space_indexes(space_name) == {"primary", "bucket_id"}
