use std::{collections, rc};

use picoplugin::system::tarantool::{fiber, net_box, tlua};

pub type ProducerInstances = rc::Rc<fiber::Mutex<collections::HashMap<String, usize>>>;
pub type ConsumerConnections = rc::Rc<
    fiber::Mutex<collections::HashMap<String, collections::HashMap<String, rc::Rc<net_box::Conn>>>>,
>;

#[derive(Debug, Clone)]
pub enum Job {
    Configure(Vec<config::VShardGroupConfig>),
    Stop {
        vshard_group: String,
        replicaset_id: String,
    },
    Start {
        config: Box<config::JobConfig>,
        topology: Vec<config::VShardGroupConfig>,
    },
    CreateWriters {
        writers_cfgs: Vec<config::WriterConfig>,
    },
}

#[allow(dead_code)]
#[derive(Default, Debug, serde::Deserialize, tlua::LuaRead)]
pub struct Meta {
    pub name: String,
    pub r#type: String,
}

#[allow(dead_code)]
#[derive(Debug, serde::Deserialize, tlua::LuaRead)]
pub struct Instance {
    pub instance_id: String,
    pub instance_uuid: String,
}

#[derive(Debug, serde::Deserialize, serde::Serialize, tlua::Push, tlua::PushInto)]
pub enum SidecarDDLType {
    Insert,
    Update,
    Delete,
    Replace,
}

#[derive(Debug, serde::Deserialize, serde::Serialize, tlua::Push, tlua::PushInto)]
pub struct SidecarDDL {
    pub space_id: u32,
    pub key: Vec<u8>,
    pub data: Vec<u8>,
    pub kind: SidecarDDLType,
    pub target_space_name: Option<String>,
}

#[derive(Debug, serde::Serialize, serde::Deserialize)]
pub struct DDL {
    pub ddl: SidecarDDL,
    pub vshard_group: String,
    pub replicaset_id: String,
}

#[derive(Debug, serde::Serialize, serde::Deserialize)]
pub struct UpdatePassword {
    pub new_pass: String,
}

#[derive(Debug, serde::Serialize, serde::Deserialize)]
pub struct SpaceInfo {
    pub name: String,
    pub space: String,
    pub indexes: Vec<String>,
}
