use once_cell::sync;
use std::{collections, sync as s_sync};

pub use config::*;

pub const SYSTEM_SPACES: [u32; 27] = [
    257, 272, 276, 277, 280, 281, 284, 285, 286, 288, 289, 296, 297, 304, 305, 312, 313, 320, 328,
    330, 340, 341, 356, 364, 372, 380, 512,
];
pub const IGNORED_SYSTEM_SPACES: [u32; 23] = [
    257, 272, 276, 277, 281, 284, 285, 286, 289, 296, 297, 305, 313, 320, 328, 330, 340, 341, 356,
    364, 372, 380, 512,
];
pub const READER_CBUS_ENDPOINT: &str = "reader_cbus_endpoint";
pub const READER_CBUS_ENDPOINT_F: &str = "reader_cbus_endpoint_f";

type SpaceMap = collections::HashMap<u32, SpaceConfig>;
type ReplicasetMap = collections::HashMap<String, SpaceMap>;
type VShardMap = sync::OnceCell<s_sync::RwLock<collections::HashMap<String, ReplicasetMap>>>;

pub static mut VSHARD_MAP: VShardMap = sync::OnceCell::new();

pub unsafe fn fill_vshard_map(groups: &[config::VShardGroupConfig]) {
    if VSHARD_MAP.get().is_none() {
        VSHARD_MAP
            .set(s_sync::RwLock::new(collections::HashMap::new()))
            .expect("set space map should not fail");
    };
    let mut vshard_map = VSHARD_MAP
        .get()
        .expect("vshard map should exist")
        .write()
        .expect("locking vshard map should not fail");
    for group in groups {
        if vshard_map.get(&group.name).is_none() {
            vshard_map.insert(group.name.clone(), collections::HashMap::new());
        }
        let replicaset_map = vshard_map
            .get_mut(&group.name)
            .expect("replicaset map shoud exist");
        for replicaset in &group.replicasets {
            if replicaset_map.get(&replicaset.uuid).is_none() {
                replicaset_map.insert(replicaset.uuid.clone(), collections::HashMap::new());
            }
            let space_map = replicaset_map
                .get_mut(&replicaset.uuid)
                .expect("space map should exist");
            for space in &replicaset.spaces {
                space_map.insert(space.space_id, space.clone());
            }
        }
    }
}
