#[derive(Debug, serde::Serialize, serde::Deserialize)]
pub enum Error {
    ReaderReplicaConnect,
    ReaderReplicaSnapshot,
    ReaderReplicaClosed,
    ReaderReplicaStopped,

    LuaPopDML,
    LuaPushDML,
    LuaApplyDML,
    LuaInitVshard,
    LuaGetState,
    LuaInsertState,
    LuaUpdateState,

    WriterLeaderConnect,
    WriterApplyDDL,
    WriterEncode,

    SqlExecute,

    Initialization(String),

    BufferCreateSpace,
    BufferCreateRecord,
    BufferGetRecords,
    BufferDeleteRecord,

    LsnQueueCreateSpace,
    LsnQueueCreateRecord,
    LsnQueueSetWritten,
}

impl From<String> for Error {
    fn from(value: String) -> Self {
        match value.as_str() {
            "reader_replica_connect" => Self::ReaderReplicaConnect,
            "reader_replica_snapshot" => Self::ReaderReplicaSnapshot,
            "reader_replica_closed" => Self::ReaderReplicaClosed,
            "reader_replica_stopped" => Self::ReaderReplicaStopped,
            "lua_pop_dml" => Self::LuaPopDML,
            "lua_push_dml" => Self::LuaPushDML,
            "lua_apply_dml" => Self::LuaApplyDML,
            "lua_init_vshard" => Self::LuaInitVshard,
            "lua_get_state" => Self::LuaGetState,
            "lua_insert_state" => Self::LuaInsertState,
            "lua_update_state" => Self::LuaUpdateState,
            "writer_leader_connect" => Self::WriterLeaderConnect,
            "writer_apply_ddl" => Self::WriterApplyDDL,
            "sql_execute" => Self::SqlExecute,
            "buffer_create_space" => Self::BufferCreateSpace,
            "buffer_create_record" => Self::BufferCreateRecord,
            "buffer_get_records" => Self::BufferGetRecords,
            "buffer_delete_record" => Self::BufferDeleteRecord,
            "lsn_queue_create_space" => Self::LsnQueueCreateSpace,
            "lsn_queue_create_record" => Self::LsnQueueCreateRecord,
            "lsn_queue_set_writed" => Self::LsnQueueSetWritten,
            val => panic!("unsupported error replicator variant: {val}"),
        }
    }
}

impl Into<String> for Error {
    fn into(self) -> String {
        match self {
            Self::ReaderReplicaConnect => String::from("reader_replica_connect"),
            Self::ReaderReplicaSnapshot => String::from("reader_replica_snapshot"),
            Self::ReaderReplicaClosed => String::from("reader_replica_closed"),
            Self::ReaderReplicaStopped => String::from("reader_replica_stopped"),
            Self::LuaPopDML => String::from("lua_pop_dml"),
            Self::LuaPushDML => String::from("lua_push_dml"),
            Self::LuaApplyDML => String::from("lua_apply_dml"),
            Self::LuaInitVshard => String::from("lua_init_vshard"),
            Self::LuaGetState => String::from("lua_get_state"),
            Self::LuaInsertState => String::from("lua_insert_state"),
            Self::LuaUpdateState => String::from("lua_update_state"),
            Self::WriterLeaderConnect => String::from("writer_leader_connect"),
            Self::WriterApplyDDL => String::from("writer_apply_ddl"),
            Self::WriterEncode => String::from("writer_encode"),
            Self::SqlExecute => String::from("sql_exute"),
            Self::BufferCreateSpace => String::from("buffer_create_space"),
            Self::BufferCreateRecord => String::from("buffer_create_record"),
            Self::BufferGetRecords => String::from("buffer_get_records"),
            Self::BufferDeleteRecord => String::from("buffer_delete_record"),
            Self::LsnQueueCreateSpace => String::from("lsn_queue_create_space"),
            Self::LsnQueueCreateRecord => String::from("lsn_queue_create_record"),
            Self::LsnQueueSetWritten => String::from("lsn_queue_set_writed"),
            Self::Initialization(msg) => msg,
        }
    }
}
