use std::fmt;

use crate::package;

/// Filter which passes packages only from sharded spaces.
pub struct Filter;

impl fmt::Display for Filter {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(f, "ShardedFilter")
    }
}

impl super::Filter for Filter {
    fn apply(&self, package: package::Package) -> package::Package {
        match &package.operation {
            package::Operation::DML(dml) => {
                if dml.meta.bucket_id.is_none() || dml.meta.bucket_id.unwrap() == 0 {
                    package::Package {
                        operation: package::Operation::DML(std::rc::Rc::new(dml.to_empty())),
                        received_at: package.received_at,
                    }
                } else {
                    package
                }
            }
            _ => package,
        }
    }
}
