use std::fmt;

use crate::package;

/// Filter which passes packages only from allowed spaces.
pub struct Filter {
    disallowed_spaces: Vec<String>,
}

impl Filter {
    pub fn new(disallowed_spaces: Vec<String>) -> Self {
        Self { disallowed_spaces }
    }
}

impl fmt::Display for Filter {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        write!(f, "SpaceFilter({:?})", self.disallowed_spaces)
    }
}

impl super::Filter for Filter {
    fn apply(&self, package: package::Package) -> package::Package {
        match &package.operation {
            package::Operation::DML(dml) => {
                if dml.meta.space_name.is_none()
                    || self
                        .disallowed_spaces
                        .contains(dml.meta.space_name.as_ref().unwrap())
                {
                    package::Package {
                        operation: package::Operation::DML(std::rc::Rc::new(dml.to_empty())),
                        received_at: package.received_at,
                    }
                } else {
                    package
                }
            }
            _ => package,
        }
    }
}
