use std::{collections, mem, rc, time};

use crate::package;

pub struct Limbo {
    buf: collections::LinkedList<(rc::Rc<package::DML>, time::Instant)>,
}

impl Limbo {
    pub fn new() -> Self {
        Self {
            buf: collections::LinkedList::new(),
        }
    }

    pub fn push(&mut self, dml: rc::Rc<package::DML>, received_at: time::Instant) {
        self.buf.push_back((dml, received_at));
    }

    pub fn take_to(
        &mut self,
        lsn: u64,
    ) -> collections::LinkedList<(rc::Rc<package::DML>, time::Instant)> {
        // This case may be reachable if there was a transaction but we do not replicate those spaces
        // So we receive commit and want to take records but buf is actually empty
        if self.buf.is_empty() {
            return self.buf.split_off(0);
        }
        let mut index = 0;
        for (idx, (dml, _received_at)) in self.buf.iter().enumerate() {
            index = idx;
            if dml.meta.lsn.unwrap() >= lsn {
                break;
            }
        }
        // Split off makes self.buff contain the left part or buffer which are elder records
        // So we make replace self.buf with this right LL which containes newer records
        let right = self.buf.split_off(index + 1);
        mem::replace(&mut self.buf, right)
    }

    pub fn take_from(
        &mut self,
        lsn: u64,
    ) -> collections::LinkedList<(rc::Rc<package::DML>, time::Instant)> {
        let mut index = 0;
        for (idx, (dml, _received_at)) in self.buf.iter().enumerate() {
            index = idx;
            if dml.meta.lsn.unwrap() >= lsn {
                break;
            }
        }
        self.buf.split_off(index)
    }
}
