use once_cell::sync;

pub static RECEIVED_PACKAGES_COUNTER: sync::Lazy<prometheus::IntCounterVec> =
    sync::Lazy::new(|| {
        prometheus::register_int_counter_vec!(
            "uroboros_received_packages_count",
            "Uroboros received packages counter",
            &["vshard_group", "replicaset_id", "replicaset_alias"]
        )
        .unwrap()
    });

pub static RECEIVED_PACKAGES_FILTERED_COUNTER: sync::Lazy<prometheus::IntCounterVec> =
    sync::Lazy::new(|| {
        prometheus::register_int_counter_vec!(
            "uroboros_received_filtered_packages_count",
            "Uroboros received filtered packages counter",
            &["vshard_group", "replicaset_id", "replicaset_alias"]
        )
        .unwrap()
    });

pub static APPLIED_PACKAGES_COUNTER: sync::Lazy<prometheus::IntCounterVec> =
    sync::Lazy::new(|| {
        prometheus::register_int_counter_vec!(
            "uroboros_applied_packages_count",
            "Uroboros applied packages counter",
            &["vshard_group", "replicaset_id", "replicaset_alias"]
        )
        .unwrap()
    });

pub static APPLIED_BYTES_COUNTER: sync::Lazy<prometheus::IntCounterVec> = sync::Lazy::new(|| {
    prometheus::register_int_counter_vec!(
        "uroboros_applied_bytes_count",
        "Uroboros applied bytes counter",
        &["vshard_group", "replicaset_id", "replicaset_alias"]
    )
    .unwrap()
});

pub static LSN_GAUGE: sync::Lazy<prometheus::GaugeVec> = sync::Lazy::new(|| {
    prometheus::register_gauge_vec!(
        "uroboros_lsn_gauge",
        "Uroboros lsn gauge",
        &["vshard_group", "replicaset_id", "replicaset_alias"]
    )
    .unwrap()
});

pub static LAG_GAUGE: sync::Lazy<prometheus::GaugeVec> = sync::Lazy::new(|| {
    prometheus::register_gauge_vec!(
        "uroboros_lag_gauge",
        "Uroboros full package lag gauge",
        &["vshard_group", "replicaset_id", "replicaset_alias"]
    )
    .unwrap()
});

pub static WORKER_RECEIVE_LAG_GAUGE: sync::Lazy<prometheus::GaugeVec> = sync::Lazy::new(|| {
    prometheus::register_gauge_vec!(
        "uroboros_worker_receive_lag_gauge",
        "Uroboros worker receive package lag gauge",
        &["vshard_group", "replicaset_id", "replicaset_alias"]
    )
    .unwrap()
});

pub static WORKER_APPLY_LAG_GAUGE: sync::Lazy<prometheus::GaugeVec> = sync::Lazy::new(|| {
    prometheus::register_gauge_vec!(
        "uroboros_worker_lag_gauge",
        "Uroboros worker apply package lag gauge",
        &["vshard_group", "replicaset_id", "replicaset_alias"]
    )
    .unwrap()
});

pub static VSHARD_APPLY_GAUGE: sync::Lazy<prometheus::GaugeVec> = sync::Lazy::new(|| {
    prometheus::register_gauge_vec!(
        "uroboros_vshard_apply_gauge",
        "Uroboros vshard apply gauge",
        &["vshard_group", "replicaset_id", "replicaset_alias"]
    )
    .unwrap()
});

/// Collects custom metrics into string buffer.
///
/// # Panics
///
/// When prometheus fails to serialize metrics
///
/// # Returns
///
/// String with metrics in prometheus format
#[must_use]
pub fn collect() -> String {
    let encoder = prometheus::TextEncoder::new();
    let metric_families = prometheus::gather();
    encoder.encode_to_string(&metric_families).unwrap()
}
