use picomole::package;

#[derive(Debug)]
pub enum Operation {
    Commit((u64, u64)),
    Rollback((u64, u64)),
}

#[derive(Debug)]
pub struct Synchro {
    pub lsn: u64,
    pub replica_id: u64,
    pub operation: Operation,
}

impl From<package::Synchro> for Synchro {
    fn from(value: package::Synchro) -> Self {
        match value {
            package::Synchro::Confirm(v) => {
                let (header, body) = v.destruct();
                Synchro {
                    lsn: header.lsn.expect("lsn must be filled for synchro"),
                    replica_id: header
                        .replica_id
                        .expect("replica_id must be filled for synchro"),
                    operation: Operation::Commit((body.replica_id, body.lsn)),
                }
            }
            package::Synchro::Rollback(v) => {
                let (header, body) = v.destruct();
                Synchro {
                    lsn: header.lsn.expect("lsn must be filled for synchro"),
                    replica_id: header
                        .replica_id
                        .expect("replica_id must be filled for synchro"),
                    operation: Operation::Rollback((body.replica_id, body.lsn)),
                }
            }
            operation => {
                panic!("syncro operation {operation:?} is not supported for anonymous replica")
            }
        }
    }
}
