use std::{rc, time};

use picoplugin::system::tarantool::fiber;

use crate::{data::state, error};

pub type WriterChannel = fiber::Channel<(rc::Rc<ReplicaInfo>, u64, ReplicaMessage, time::Instant)>;
pub type BucketsChannels = Vec<WriterChannel>;

#[derive(Clone, Debug, serde::Serialize, serde::Deserialize)]
pub struct ReplicaInfo {
    pub vshard_group: String,
    pub replicaset_uuid: String,
    pub replicaset_alias: Option<String>,
    pub replica_uuid: String,
}

#[derive(Clone, Debug, Default, serde::Serialize, serde::Deserialize)]
pub struct State {
    pub vclock: picomole::VClock,
    pub cluster_uuid: uuid::Uuid,
    pub replica_uuid: uuid::Uuid,
}

impl State {
    pub fn select(cluster_uuid: &str) -> Result<Option<Self>, error::Error> {
        log::info!("prepare state for cluster: {cluster_uuid}");
        match state::select(cluster_uuid)? {
            None => Ok(None),
            Some(rows) => {
                let mut vclock = picomole::VClock::with_capacity(rows.len());
                let cluster_uuid = uuid::Uuid::parse_str(&rows.first().unwrap().cluster_uuid)
                    .expect("parsing cluster_uuid should not fail");
                let replica_uuid = uuid::Uuid::parse_str(&rows.first().unwrap().replica_uuid)
                    .expect("parsing replica_uuid should not fail");
                for v in rows {
                    vclock.insert(v.replica_id, v.lsn);
                }
                Ok(Some(Self {
                    vclock,
                    cluster_uuid,
                    replica_uuid,
                }))
            }
        }
    }
}

#[derive(Debug, Clone)]
pub enum ReplicaMessage {
    State(fiber::Channel<()>),
    DML(rc::Rc<crate::package::DML>),
}
